(* Mathematica package *)
(* Created by the Wolfram Workbench 14.04.2016 *)


BeginPackage[
	"QuantumAlgebra`Styles`",
	{
		"QuantumAlgebra`",
		"Notation`"
	}
]


(* Exported symbols added here with SymbolName::usage *) 

SetNotebookStyles::usage = 
	"SetNotebookStyles[] defines Notebook styles for the InputNotebook. \n" <>
	"SetNotebookStyles[nb] defines Notebook styles for the specified notebook. \n" <>
	"Calling this function once is required for proper display of the input / output " <> 
	"elements in Dirac Bra-Ket Notation.\n" <>
	"The original stylesheet is modified by adding some styles, but not changed completely."

setDocumentationStyles::usage = 
	"setDocumentationStyles[options] sets the documentation styles suitable for the " <>
	"QuantumAlgebra application's documentation so that Quantum objects can be displayed " <>
	"properly. This is usually not intended for the user."

App::usage = 
	"App option is used to overwrite the default."

Doc::usage = 
	"Doc option is used to overwrite the default."

FileSpec::usage = 
	"FileSpec option is used to overwrite the default."

Begin["`Private`"]
(* Implementation of the package *)


(***** Set Notebook Styles *****)

SetNotebookStyles[doc_NotebookObject] :=
	Module[{defaultStyles, QAStyles, ds},

		defaultStyles[] :=
			Module[{sd},
				sd = CurrentValue[doc, StyleDefinitions];
				
				If[Count[sd, "Reference.nb", 10] > 0,
					If[Count[sd, _Cell, 10] > 0,
						Return[Cases[sd, {cells__Cell} -> cells, 10]],
						Return["Reference.nb"]
					]
				];
				
				If[StringQ[sd],
					Return[sd]];
				
				sd = Cases[sd, (StyleDefinitions->sym_String)->sym, 10];
				If[ListQ[sd] && Length[sd] > 0 && StringQ[sd[[1]]],
					Return[sd[[1]]]];
				
				"Default.nb"
			];
		
		ds = defaultStyles[];		
		QAStyles :=
		{
			If[ds === "Reference.nb",
				Cell[StyleData[StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]],
				Cell[StyleData[StyleDefinitions -> ds]]
			],
			Cell["Bra Ket Styles","Section"],
			Cell["\< The cells below define certain styles needed for defining Bras&Kets.These styles serve to give the correct structural properties to Bras & Kets \>","Text"],
			Cell[StyleData["BraKetArg"],
				AutoStyleOptions->{"UnmatchedBracketStyle"->"UnmatchedBracket"},
				SpanMinSize->Automatic,
				StyleMenuListing->None,
				AdjustmentBoxOptions->{BoxMargins->{{0,0},{0,0}}},
				TagBoxOptions->{Editable->True,Selectable->True}],
			Cell[StyleData["BraWrapper"],
				AutoStyleOptions->{"UnmatchedBracketStyle"->None},
				SpanMinSize->2,
				ScriptBaselineShifts->{0.6,0.9},
				StyleMenuListing->None,
				AdjustmentBoxOptions->{BoxMargins->{{0.2,-0.13},{0,0}}},
				TagBoxOptions->{Editable->False}],
			Cell[StyleData["KetWrapper"],
				AutoStyleOptions->{"UnmatchedBracketStyle"->None},
				SpanMinSize->2,
				ScriptBaselineShifts->{0.6,0.9},
				StyleMenuListing->None,
				AdjustmentBoxOptions->{BoxMargins->{{-0.1,0.2},{0,0}}},
				TagBoxOptions->{Editable->False}],
			Cell[StyleData["BraKetWrapper"],
				AutoStyleOptions->{"UnmatchedBracketStyle"->None},
				SpanMinSize->2,
				ScriptBaselineShifts->{0.6,0.9},
				StyleMenuListing->None,
				AdjustmentBoxOptions->{BoxMargins->{{0.25,0.25},{0,0}}},
				TagBoxOptions->{Editable->False}],
			Cell[StyleData["RIndex"],
				StyleMenuListing->None,
				AdjustmentBoxOptions->{BoxMargins->{{-0.16,0},{0,0}}},
				TagBoxOptions->{Editable->False}],
			Cell[StyleData["LIndex"],
				StyleMenuListing->None,
				AdjustmentBoxOptions->{BoxMargins->{{0,-0.1},{0,0}}},
				TagBoxOptions->{Editable->False}]
		};
		
		Which[
			StringQ[ds] && ds === "Reference.nb",			(* CASE 1: REFERENCE without cells *)
			Print["A (PROBABLY) GUIDE"];
			SetOptions[
				doc,
				StyleDefinitions->Notebook[
					QAStyles,
					StyleDefinitions->"Default.nb"]],
			
			StringQ[ds],									(* CASE 2: NORMAL case *)
			SetOptions[
				doc,
				StyleDefinitions->Notebook[
					QAStyles,
					StyleDefinitions->ToFileName[
						{
							$InstallationDirectory, 
							"SystemFiles", 
							"FrontEnd", 
							"StyleSheets"
						}, 
						"PrivateStylesheetFormatting.nb"]]
			],
			
			True,											(* CASE 3: REST: REFERENCE with cells *)
			SetOptions[
				doc,
				StyleDefinitions->Notebook[
					Join[ds, Drop[QAStyles, 1]],
					StyleDefinitions->"Default.nb"]]
			]
	]

SetNotebookStyles[]:=
	Module[
		{nb},
		nb := InputNotebook[];
		SetNotebookStyles[nb]
	]


Options[setDocumentationStyles]=
{
	App -> {$UserDocumentsDirectory, "Programmieren", "Mathematica", "Applications" },
	Doc -> {"QuantumAlgebra", "build"},
	FileSpec -> "*.nb"
}

setDocumentationStyles[OptionsPattern[]] :=
	Module[{setStyles,app,doc,dir,fileSpec,nbs},
	
		setStyles[nb_String] :=
			Module[{nbo},
				nbo = NotebookOpen[nb, Saveable->True];
				SetNotebookStyles[nbo];
				SetOptions[nbo, Saveable->False];
				NotebookSave[nbo];
				NotebookClose[nbo];
				Print /@ StringCases[nb, ___~~"\\"~~fn:__~~".nb" -> (fn <> ".nb")];
		];
		
		app = OptionValue[App];
		doc = OptionValue[Doc];
		fileSpec = OptionValue[FileSpec];
		dir = FileNameJoin[Flatten[{app, doc}]];
		nbs = FileNames[fileSpec, {dir}, 10];
		Map[setStyles, nbs];
		True
	]

(*
Alternative Options
{
	App \[Rule] {$UserBaseDirectory, "Applications"},
	Doc \[Rule] {"QuantumAlgebra", "Documentation"},
}
*)

End[]

EndPackage[]
