(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 10.2' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       158,          7]
NotebookDataLength[      9234,        292]
NotebookOptionsPosition[      4480,        164]
NotebookOutlinePosition[      7540,        234]
CellTagsIndexPosition[      7497,        231]
WindowTitle->QuantumAlgebra Modules - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[" ", "GuideColorBar",
 CellMargins->{{Inherited, Inherited}, {-5, 0}}],

Cell[TextData[{
 ButtonBox["Mathematica",
  BaseStyle->{"Link", "LinkTrail"},
  ButtonData->"paclet:QuantumAlgebra/guide/QuantumAlgebra"],
 StyleBox[" > ", "LinkTrailSeparator"]
}], "LinkTrail"],

Cell[CellGroupData[{

Cell["QuantumAlgebra Modules", "GuideTitle",
 CellID->942062912],

Cell["\<\
This guide page gives an overview about the modules in the Quantum Algebra \
package and their content.\
\>", "GuideAbstract",
 CellID->2001916300],

Cell[CellGroupData[{

Cell["QuantumAlgebra.m", "GuideFunctionsSubsection",
 CellID->168525014],

Cell["\<\
This is the main module of the package and the only one in the original \
package. 
It contains the Algebra rules for Quantum Algebra, i.e. how the basic Quantum \
objects behave and are simplified in calculations. A few helper functions \
help decide what type of basic Quantum object an expression is.
This release of the Quantum Algebra package contains a few improvements via \
the original package: the decision about the type of Quantum object is \
improved and more detailed, a few bugs are fixed and a new type of constant - \
called Complex Constant - has been introduced to permit further \
simplification of Quantum expressions.\
\>", "GuideText",
 CellID->165460465],

Cell["\t", "GuideDelimiterSubsection",
 CellID->22563968]
}, Open  ]],

Cell[CellGroupData[{

Cell["Analyze.m", "GuideFunctionsSubsection",
 CellID->410453794],

Cell["\<\
Contains checks for Quantum types like Ket, Bra, BraKet, Operator, \
SimpleConstant and QuantumConstant. They permit a detailed check, if an \
expression is a supported Quantum object.\
\>", "GuideText",
 CellID->33873020],

Cell["\t", "GuideDelimiterSubsection",
 CellID->6369384]
}, Open  ]],

Cell[CellGroupData[{

Cell["Options.m", "GuideFunctionsSubsection",
 CellID->306482911],

Cell["\<\
Supports some options for the package. There are 3 options for printout \
control of diagnostic prints and 1 option to control the system of Quantum \
types (DefinitionLevel).\
\>", "GuideText",
 CellID->174454626],

Cell["\t", "GuideDelimiterSubsection",
 CellID->662756017]
}, Open  ]],

Cell[CellGroupData[{

Cell["QuantumNotation.m", "GuideFunctionsSubsection",
 CellID->6375819],

Cell["\<\
Contains functions for the definition of Notations, Input Aliases, Boxes. \
They allow proper display of Quantum objects and 2-dimensional input.
This module replaces a definition Notebook (QANotations.nb) in the original \
package.\
\>", "GuideText",
 CellID->38417311],

Cell["\t", "GuideDelimiterSubsection",
 CellID->732978]
}, Open  ]],

Cell[CellGroupData[{

Cell["Styles.m", "GuideFunctionsSubsection",
 CellID->103788340],

Cell["\<\
Contains functions for the definition of Notebook styles. These styles \
control the proper display of Quantum objects in Notebooks and in the \
documentation.
This is a partial replacement of the original definition Notebook \
(QANotations.nb)\
\>", "GuideText",
 CellID->710547200]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["TUTORIALS", "GuideTutorialsSection",
 CellID->415694126],

Cell[TextData[{
 ButtonBox["Quantum Algebra Overview",
  BaseStyle->"Link",
  ButtonData->"paclet:QuantumAlgebra/tutorial/QuantumAlgebraOverview"],
 " - Overview page with links to all documentation pages"
}], "GuideTutorial",
 CellID->1885805579],

Cell[TextData[ButtonBox["Quantum Algebra Basics",
 BaseStyle->"Link",
 ButtonData->
  "paclet:QuantumAlgebra/tutorial/QuantumAlgebraBasics"]], "GuideTutorial",
 CellID->806871991]
}, Open  ]],

Cell[CellGroupData[{

Cell["MORE ABOUT", "GuideMoreAboutSection",
 CellID->23220180],

Cell[TextData[ButtonBox["Quantum Algebra Guide",
 BaseStyle->"Link",
 ButtonData->
  "paclet:QuantumAlgebra/guide/QuantumAlgebraGuide"]], "GuideMoreAbout",
 CellID->1567025153]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"QuantumAlgebra Modules - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, 
  "Metadata" -> {
   "built" -> "{2016, 5, 31, 18, 14, 40.2351702}", "context" -> 
    "QuantumAlgebra`", "keywords" -> {}, "index" -> True, "label" -> 
    "Quantum Algebra Package Guide", "language" -> "en", "paclet" -> 
    "QuantumAlgebra Package", "status" -> "None", "summary" -> 
    "This guide page gives an overview about the modules in the Quantum \
Algebra package and their content.", "synonyms" -> {}, "title" -> 
    "QuantumAlgebra Modules", "type" -> "Guide", "uri" -> 
    "QuantumAlgebra/guide/QuantumAlgebraModules"}, "LinkTrails" -> "", 
  "SearchTextTranslated" -> ""},
FrontEndVersion->"10.2 for Microsoft Windows (64-bit) (July 7, 2015)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell["Bra Ket Styles", "Section"], 
   Cell[" The cells below define certain styles needed for defining \
Bras&Kets.These styles serve to give the correct structural properties to \
Bras & Kets ", "Text"], 
   Cell[
    StyleData["BraKetArg"], SpanMinSize -> Automatic, StyleMenuListing -> 
    None, AdjustmentBoxOptions -> {BoxMargins -> {{0, 0}, {0, 0}}}, 
    TagBoxOptions -> {Editable -> True, Selectable -> True}, 
    AutoStyleOptions -> {"UnmatchedBracketStyle" -> "UnmatchedBracket"}], 
   Cell[
    StyleData["BraWrapper"], ScriptBaselineShifts -> {0.6, 0.9}, SpanMinSize -> 
    2, StyleMenuListing -> None, 
    AdjustmentBoxOptions -> {BoxMargins -> {{0.2, -0.13}, {0, 0}}}, 
    TagBoxOptions -> {Editable -> False}, 
    AutoStyleOptions -> {"UnmatchedBracketStyle" -> None}], 
   Cell[
    StyleData["KetWrapper"], ScriptBaselineShifts -> {0.6, 0.9}, SpanMinSize -> 
    2, StyleMenuListing -> None, 
    AdjustmentBoxOptions -> {BoxMargins -> {{-0.1, 0.2}, {0, 0}}}, 
    TagBoxOptions -> {Editable -> False}, 
    AutoStyleOptions -> {"UnmatchedBracketStyle" -> None}], 
   Cell[
    StyleData["BraKetWrapper"], ScriptBaselineShifts -> {0.6, 0.9}, 
    SpanMinSize -> 2, StyleMenuListing -> None, 
    AdjustmentBoxOptions -> {BoxMargins -> {{0.25, 0.25}, {0, 0}}}, 
    TagBoxOptions -> {Editable -> False}, 
    AutoStyleOptions -> {"UnmatchedBracketStyle" -> None}], 
   Cell[
    StyleData["RIndex"], StyleMenuListing -> None, 
    AdjustmentBoxOptions -> {BoxMargins -> {{-0.16, 0}, {0, 0}}}, 
    TagBoxOptions -> {Editable -> False}], 
   Cell[
    StyleData["LIndex"], StyleMenuListing -> None, 
    AdjustmentBoxOptions -> {BoxMargins -> {{0, -0.1}, {0, 0}}}, 
    TagBoxOptions -> {Editable -> False}]}, Visible -> False, FrontEndVersion -> 
  "10.2 for Microsoft Windows (64-bit) (July 7, 2015)", StyleDefinitions -> 
  "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[616, 21, 75, 1, 3, "GuideColorBar"],
Cell[694, 24, 194, 5, 19, "LinkTrail"],
Cell[CellGroupData[{
Cell[913, 33, 64, 1, 47, "GuideTitle",
 CellID->942062912],
Cell[980, 36, 157, 4, 48, "GuideAbstract",
 CellID->2001916300],
Cell[CellGroupData[{
Cell[1162, 44, 72, 1, 30, "GuideFunctionsSubsection",
 CellID->168525014],
Cell[1237, 47, 688, 12, 137, "GuideText",
 CellID->165460465],
Cell[1928, 61, 57, 1, 21, "GuideDelimiterSubsection",
 CellID->22563968]
}, Open  ]],
Cell[CellGroupData[{
Cell[2022, 67, 65, 1, 30, "GuideFunctionsSubsection",
 CellID->410453794],
Cell[2090, 70, 232, 5, 34, "GuideText",
 CellID->33873020],
Cell[2325, 77, 56, 1, 21, "GuideDelimiterSubsection",
 CellID->6369384]
}, Open  ]],
Cell[CellGroupData[{
Cell[2418, 83, 65, 1, 30, "GuideFunctionsSubsection",
 CellID->306482911],
Cell[2486, 86, 224, 5, 34, "GuideText",
 CellID->174454626],
Cell[2713, 93, 58, 1, 21, "GuideDelimiterSubsection",
 CellID->662756017]
}, Open  ]],
Cell[CellGroupData[{
Cell[2808, 99, 71, 1, 30, "GuideFunctionsSubsection",
 CellID->6375819],
Cell[2882, 102, 280, 6, 58, "GuideText",
 CellID->38417311],
Cell[3165, 110, 55, 1, 21, "GuideDelimiterSubsection",
 CellID->732978]
}, Open  ]],
Cell[CellGroupData[{
Cell[3257, 116, 64, 1, 30, "GuideFunctionsSubsection",
 CellID->103788340],
Cell[3324, 119, 293, 7, 58, "GuideText",
 CellID->710547200]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[3666, 132, 62, 1, 21, "GuideTutorialsSection",
 CellID->415694126],
Cell[3731, 135, 247, 6, 19, "GuideTutorial",
 CellID->1885805579],
Cell[3981, 143, 179, 4, 70, "GuideTutorial",
 CellID->806871991]
}, Open  ]],
Cell[CellGroupData[{
Cell[4197, 152, 62, 1, 70, "GuideMoreAboutSection",
 CellID->23220180],
Cell[4262, 155, 176, 4, 70, "GuideMoreAbout",
 CellID->1567025153]
}, Open  ]],
Cell[4453, 162, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)
